/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_XMLIO_H
#define GWF_XMLIO_H

#ifdef __cplusplus
extern "C" {
#endif

#include "gwf_project.h"
#include "gwf_projlist.h"
#include "gwf_error.h"
#include <gtk/gtk.h>
#include <tree.h>
#include <parser.h>

#define	XML_FILE_VERSION		"0.4.0"
#define XML_HREF			"http://gwf.sourceforge.net"

typedef struct	_GwfXmlIO	GwfXmlIO;
typedef struct	_GwfXmlLoader	GwfXmlLoader;
typedef struct	_GwfXmlSaver	GwfXmlSaver;
typedef struct	_XmlSaveData	XmlSaveData;

typedef gint	(*GwfXmlLoadFunc)	(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error);
typedef gint	(*GwfXmlSaveFunc)	(xmlNodePtr parent,xmlNsPtr ns,GwfProject *project);

struct _GwfXmlLoader
{
   gchar		*name;
   gchar		*version;
   gchar		*href;
   GwfXmlLoadFunc	func;
};

struct _GwfXmlSaver
{
   gchar		*name;
   gchar		*version;
   gchar		*href;
   GwfXmlSaveFunc	func;
};

struct _GwfXmlIO
{
   xmlChar		*buffer;
   
   /* Root of the document tree */
   xmlDocPtr		doc;
   
   /* Additional load & save functions */
   GHashTable		*loaders;
   GHashTable		*savers;
};

struct _XmlSaveData
{
  GwfProject	*project;
  xmlNodePtr	parent;
  xmlNsPtr	ns;
  gint		result;
};

GwfXmlIO*	gwf_xmlio_new		(void);
void		gwf_xmlio_free		(GwfXmlIO *xml_io);

gint		gwf_xmlio_add_loader	(GwfXmlIO *xml_io,gchar *name,gchar *version,gchar *href,GwfXmlLoadFunc func);
void		gwf_xmlio_del_loader	(GwfXmlIO *xml_io,gchar *name);

gint		gwf_xmlio_add_saver	(GwfXmlIO *xml_io,gchar *name,gchar *version,gchar *href,GwfXmlSaveFunc func);
void		gwf_xmlio_del_saver	(GwfXmlIO *xml_io,gchar *name);

void		gwf_xmlio_set_buffer	(GwfXmlIO *xml_io,guchar *buffer);
guchar*		gwf_xmlio_get_buffer	(GwfXmlIO *xml_io);

GwfProject*	gwf_xmlio_load		(GwfXmlIO *xml_io,GwfProjlist *projlist,GwfError *error);
gint		gwf_xmlio_save		(GwfXmlIO *xml_io,GwfProject *project);

#ifdef __cplusplus
}
#endif

#endif

